module.exports = function(grunt){
    var utilPath = 'assets/js/Util/**/*.js';
    var servicePath = 'assets/js/Service/**/*.js';
    var controllerPath = 'assets/js/Controller/**/*.js';
    var libFiles = [
        'zhuaclfrontend/assets/dist/main.js',

        'zh-interval-component/dist/zh-interval-component.js',
        'zh-grid-navigation/dist/GridNavigationLibrary.js',
        'zh-memory-storage/src/main.js',
        "zh-cache-bypass/src/CacheBypass.js",
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-reports/dist/zhReports.js',
        'zh-interval-component/dist/zh-interval-component.js',

        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        'zh-grid-configuration/dist/zh-grid-configuration.js',

        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js',
        'gitQueue/dist/main.min.js'
    ];

    grunt.initConfig({
        // grunt-contrib-concat
        concat: {
            js: {
                options: {
                    process: function(src, filePath){
                        return '\n' + '// FILE: ' + filePath + '\n' + src;
                    }
                },
                src: [
                    utilPath, servicePath, controllerPath
                ],
                dest: 'assets/js/app.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'assets/js/libs.js'
            }
        },

        // grunt-contrib-jshint
        jshint: {
            options: { 
               reporterOutput : '' 
            },
            all: ['Gruntfile.js', './assets/js/Util/**/*.js', './assets/js/Service/**/*.js', './assets/js/Controller/**/*.js']
        },

        // grunt-contrib-uglify
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/js/app.min.js': [utilPath, servicePath, controllerPath]
                }
            }
        },

        // grunt-contrib-watch
        watch: {
            pivotal: {
                files: [utilPath, servicePath, controllerPath],
                tasks: ['jshint', 'uglify', 'concat:js']
            }
        },

        // grunt-zh-id-generator
        zhIdGenerator: {
            options: {
                jsonPath: ["./assets/json"],
                menuPath: ["./assets/json/menu.json"],
                replaceIds: false
            }
        },

        // grunt-zhuacl-structure
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },

        buildEnvironment: {
            development: {
                configFile: "./config/devEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            srcFolders: ["../backend/config/", "../backend/query/", "./assets/js", "./assets/json", "./config", "./", "./assets", "../version/"]
        },

        zhWords: {
            options: {
                serverUrl: "https://rc-translate.teknisa.com/backend/index.php",
                wordsPath: "./assets/json/words/",
                organization: "20",
                project: "22683"
            }
        }
    });

    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-grid-navigation/dist/style.css');
        paths.push('bower_components/zh-reports/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-preferences/dist/style/style.css');
        paths.push('bower_components/zh-preferences/dist/style/colors/styles-inflightor.css');
        return paths;
    }

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zh-words');

    grunt.registerTask('default', ['jshint', 'uglify', 'concat', 'buildEnvironment', 'buildAclStruct', 'zhWords']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
    grunt.registerTask('basic', ['uglify', 'concat:js', 'zhIdGenerator']);
};